/// @addtogroup auxil
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file CRef.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _CREF_H_
#define _CREF_H_

#include <stdexcept>

template<typename T>
class CRef;

template<typename T>
class CRef{
  private:
    T* ptr;
    int* cnt;
    void remove() {
      if (cnt && (--*cnt == 0)){
        delete cnt;
        if(ptr) delete ptr;
      }
    }
  public:
    explicit CRef():ptr(0),cnt(0){}
    explicit CRef(T* p):ptr(p),cnt(new int(1)){}
    CRef(const CRef<T>& p)throw():ptr(p.ptr),cnt(p.cnt){
      if(cnt) ++*cnt;
    }
    ~CRef()throw(){
      remove();
    }
    CRef<T>& operator=(const CRef<T>& p)throw(){
      if(this != &p){
        remove();
        ptr=p.ptr;
        cnt = p.cnt;
        if(cnt) ++*cnt;
      }
      return *this;
    }
    T& operator()() const throw(std::domain_error){
      if(ptr) return *ptr;
      throw std::domain_error("CRef::operator() : null reference");
    }
    T& operator*() const throw(std::domain_error){
      if(ptr) return *ptr;
      throw std::domain_error("CRef::operator* : null reference");
    }
    int count() const throw(std::domain_error){
      if(cnt) return *cnt;
      throw std::domain_error("CRef::count : null reference");
    }
    friend std::ostream& operator<<(std::ostream& out,const CRef<T>& A_CRef){
      out << "CRef pointing at address " << std::hex << A_CRef.ptr;
      out << " with count=" << std::dec << *A_CRef.cnt << " ";
      return out;
    }
};
#endif /*_CREF_H_*/

/// @}

