/// @addtogroup matrixAlgorithms
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file EuclBitSet_NeighbIterator.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#if !defined(_EUCLBITSETT_NEIGHBITERATOR_H_)
#define _EUCLBITSETT_NEIGHBITERATOR_H_

/*=======================================================================*/
// NeighbIterator is a EuclBitSetT iterator
// intended for scan of the neighborhood of a given point
//
// This is the specialization for <unsigned long int,3> !!!

template<>
class EuclBitSetT<BitSetT<BitmapT<unsigned long int> >,3>::NeighbIterator : public EuclBitSetT<BitSetT<BitmapT<unsigned long int> >,3>::BitIterator{
  friend class EuclBitSetT<unsigned long int,3>;
protected:
  unsigned char i0,i1,i2; // locCoord takes values in {0,1,2} only
public:
  NeighbIterator(const EuclBitSetT& A_s, const EuclBitSetT<BitSetT<BitmapT<unsigned long int> >,3>::BitIterator& A_it):
    EuclBitSetT<BitSetT<BitmapT<unsigned long int> >,3>::BitIterator(A_s,A_it),i0(0),i1(0),i2(0){}
  void operator++(){
    if(++i0<3){
      (*this)+=this->baseEuclBitSet()->neighbStride[0];
      return;
    }else{
      i0=0;
    }
    if(++i1<3){
      (*this)+=this->baseEuclBitSet()->neighbStride[1];
      return;
    }else{
      i1=0;
    }
    if(++i2<3){
      (*this)+=this->baseEuclBitSet()->neighbStride[2];
      return;
    }else{
      i2=0;
    }
    ++(*this);
  }
};

/*=======================================================================*/
// NeighbIterator is a EuclBitSetT iterator
// intended for scan of the neighborhood of a given point

template <typename P_BitSet,int dim>
class EuclBitSetT<P_BitSet,dim>::NeighbIterator : public EuclBitSetT<P_BitSet,dim>::BitIterator{
  friend class EuclBitSetT<P_BitSet,dim>;
protected:
  unsigned char locCoord[dim]; // locCoord takes values in {0,1,2} only
public:
  NeighbIterator(const EuclBitSetT& A_s, const EuclBitSetT<P_BitSet,dim>::BitIterator& A_it):
    EuclBitSetT<P_BitSet,dim>::BitIterator(A_s,A_it)
  {
    for(int i=0;i<dim;++i) locCoord[i]=0;
  }
  void operator++(){
    for(int i=0;i<dim;++i){
      if(++locCoord[i]<3){
        (*this)+=this->baseEuclBitSet()->neighbStride[i];
        return;
      }else{
        locCoord[i]=0;
      }
    }
    ++(*this);
  }
};
#endif //_EUCLBITSETT_NEIGHBITERATOR_H_


/*=======================================================================*/
/// @}

