/// @addtogroup homology
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file cube.h
///
/// @author Pawel Pilarczyk
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2007 by Pawel Pilarczyk.
//
// This file is part of the Homology Library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

// Started in January 2002. Last revision: October 25, 2005.


#ifndef _CAPD_CUBE_H_
#define _CAPD_CUBE_H_

#include "homology/config.h"
#include "homology/textfile.h"
#include "homology/pointset.h"
#include "homology/chains.h"
#include "homology/bitfield.h"
#include "homology/integer.h"
#include "homology/hashsets.h"
#include "homology/gcomplex.h"
#include "homology/pointbas.h"
#include "homology/cubemain.h"
#include "homology/cubebase.h"
#include "homology/cubefix.h"
#include "homology/cubevar.h"

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <cstring>

namespace capd {
namespace homology {


// This header file includes three different templates for the Cube class:
// - with the use of PointBase (saves memory for overlapping cub. sets),
// - with dynamical memory allocation (most flexible, but slow),
// - with the dimension fixed as a template parameter (the fastest).

// the default cube type
typedef tCubeBase<coordinate> Cube;

// alternative names for a cube
typedef Cube FullCube;
typedef Cube HyperCube;

// structures based on cubes and cubical cells
typedef hashedset<Cube> SetOfCubes;
typedef mvmap<Cube,Cube> CombinatorialMultivaluedMap;

// some abbreviations or lower-case versions of the data type names
// (mainly for backwards compatibility)
typedef Cube cube;
typedef SetOfCubes cubes;
typedef CombinatorialMultivaluedMap cubicalmap;

template <class coordtype>
inline std::ostream &operator << (std::ostream &out,
	const tCubeBase<coordtype> &c)
{
	return WriteCube (out, c);
} /* operator << */

template <int dimfix, class coordtype>
inline std::ostream &operator << (std::ostream &out,
	const tCubeFix<dimfix,coordtype> &c)
{
	return WriteCube (out, c);
} /* operator << */

template <class coordtype>
inline std::ostream &operator << (std::ostream &out,
	const tCubeVar<coordtype> &c)
{
	return WriteCube (out, c);
} /* operator << */

template <class coordtype>
inline std::istream &operator >> (std::istream &in, tCubeBase<coordtype> &c)
{
	return ReadCube (in, c);
} /* operator >> */

template <int dimfix, class coordtype>
inline std::istream &operator >> (std::istream &in,
	tCubeFix<dimfix,coordtype> &c)
{
	return ReadCubeFix (in, c, dimfix);
} /* operator >> */

template <class coordtype>
inline std::istream &operator >> (std::istream &in, tCubeVar<coordtype> &c)
{
	return ReadCube (in, c);
} /* operator >> */


} // namespace homology
} // namespace capd


#endif // _CUBE_H_

/// @}
