//////////////////////////////////////////////////////////////////////////////
//   Package:          CAPD
///   Subpackage:
/// @addtogroup intervals
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file intervalFiendInternal.h
//
/// Interval Arithmetics - friend functions defined inside Interval class 
//
/// @author Tomasz Kapela   @date 11-01-2006
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Tomasz Kapela 2006
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.




//////////////////////////////////////////////////////////////////////////
//  RELATIONS between interval and types which can be converted to BoundType 
//////////////////////////////////////////////////////////////////////////


///  operator == (interval, scalar)
friend inline bool operator == (const Interval & A_iVal1, 
                                const T_Bound & A_Val2)
{
  return ((A_iVal1.leftBound() == (A_Val2)) && 
          (A_iVal1.rightBound() == (A_Val2)));
}

///  operator == (scalar, interval)
friend inline bool operator == (const T_Bound & A_Val1,
                                const Interval & A_iVal2)
{
   return ((A_Val1 == A_iVal2.leftBound()) && 
           (A_Val1 == A_iVal2.rightBound()));
}

///  operator !=  (interval, scalar)
friend inline bool operator != (const Interval & A_iVal1, 
                                const T_Bound & A_Val2)
{
  return ((A_iVal1.leftBound() != A_Val2) ||
          (A_iVal1.rightBound() != A_Val2));
}

///  operator !=   (scalar, interval)
friend inline bool operator != (const T_Bound & A_Val1,
                                const Interval & A_iVal2)
{
   return ((A_Val1 != A_iVal2.leftBound()) ||
           (A_Val1 != A_iVal2.rightBound()));
}

///  operator >  (interval, scalar)
friend inline bool operator > (const Interval & A_iVal1, 
                               const T_Bound & A_Val2)
{
   return (A_iVal1.leftBound() > A_Val2);
}

///  operator >   (scalar, interval)
friend inline bool operator > (const T_Bound & A_Val1,
                               const Interval & A_iVal2)
{
   return (A_Val1 > A_iVal2.rightBound());
}

///  operator >= (interval, scalar)
friend inline bool operator >= (const Interval & A_iVal1, 
                                const T_Bound & A_Val2)
{
   return (A_iVal1.leftBound() >= A_Val2);
}

///  operator >=  (scalar, interval)
friend inline bool operator >= (const T_Bound & A_Val1,
                                const Interval & A_iVal2)
{
   return (A_Val1 >= A_iVal2.rightBound());
}

///  operator  < (interval, scalar)
friend inline bool operator < (const Interval & A_iVal1, 
                               const T_Bound & A_Val2)
{
   return (A_iVal1.rightBound() < A_Val2);
}

///  operator  <  (scalar, interval)
friend inline bool operator < (const T_Bound & A_Val1,
                               const Interval & A_iVal2)
{
   return (A_Val1 < A_iVal2.leftBound());
}

/// operator <=  (interval, scalar)
friend inline bool operator <= (const Interval & A_iVal1, 
                                const T_Bound & A_Val2)
{
   return (A_iVal1.rightBound() <= A_Val2);
}

/// operator <=  (scalar, interval)
friend inline bool operator <= (const T_Bound & A_Val1, 
                                const Interval & A_iVal2)
{
   return (A_Val1 <= A_iVal2.leftBound());
}

//////////////////////////////////////////////////////////////////////////
//  ARITHMETIC OPERATORS between interval and BoundType 
//////////////////////////////////////////////////////////////////////////
 
 
/// operator +  (interval, scalar)
friend inline Interval operator+(const Interval & A_iVal, 
                                 const T_Bound &A_x)
{  
  return add(A_iVal, A_x);
}

/// operator + (scalar, interval)
friend inline Interval operator+(const T_Bound & A_x, 
                                 const Interval & A_iVal)
{ 
  return add(A_iVal, A_x);
}

/// operator -  (interval, scalar)
friend inline Interval operator-(const Interval & A_iVal, 
                                 const T_Bound &A_x)
{  
  return substract(A_iVal, A_x);
}
  
/// operator - (scalar, interval)
friend inline Interval operator-(const T_Bound  &A_x, 
                                 const Interval & A_iVal)
{ 
  return substract(A_x, A_iVal);
}
  
/// operator * (interval, scalar)
friend inline Interval operator*(const Interval& A_iVal, 
                                 const T_Bound& A_x)
{
  return multiply(A_iVal, A_x);
}

/// operator * (scalar, interval) 
friend inline Interval operator*(const T_Bound& A_x, 
                                 const Interval& A_iVal)
{
  return multiply(A_iVal, A_x);
}
  
/// operator / (scalar, interval)
friend inline Interval operator/(const T_Bound& A_x, 
                                 const Interval& A_iVal)
{
  return  divide(A_x, A_iVal);
}
  
/// operator / (interval, scalar)
friend inline Interval operator/(const Interval& A_iVal, 
                                 const T_Bound& A_x)
{
  return divide (A_iVal, A_x);
}

// @}
