/// @addtogroup vectalg
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file rowVector.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _RowVector_H
#define _RowVector_H

#include <ostream>

template<typename Scalar, int rows, int cols>
class Matrix;

template<typename Scalar,int capacity>
class Vector;

template<typename Scalar, int cols>
class RowVector
{
public:
   typedef Scalar ScalarType;
   typedef ScalarType* iterator;
   typedef const ScalarType* const_iterator;
   typedef RowVector<Scalar,cols> VectorType;

   RowVector(const Scalar* a_pointer, int a_dim)
      : m_pointer(const_cast<Scalar*>(a_pointer)),
        m_dim(a_dim)
   {}
   RowVector& operator=(const RowVector&);
   RowVector& operator=(const Vector<Scalar,cols>&);
   RowVector& operator+=(const RowVector&);
   RowVector& operator+=(const Vector<Scalar,cols>&);
   RowVector& operator-=(const RowVector&);
   RowVector& operator-=(const Vector<Scalar,cols>&);
   operator Vector<Scalar,cols>() const;

   inline Scalar& operator[](int a_col)
   {
      return *(m_pointer+a_col);
   }
   inline const Scalar& operator[](int a_col) const
   {
      return *(m_pointer+a_col);
   }

   Scalar euclNorm() const;
   bool normalize();
   void clear();
   inline int dimension() const
   {
      return m_dim;
   }
   inline iterator begin()
   {
      return iterator(m_pointer);
   }
   inline iterator end()
   {
      return iterator(m_pointer+m_dim);
   }
   inline const_iterator begin() const
   {
      return const_iterator(m_pointer);
   }
   inline const_iterator end() const
   {
      return const_iterator(m_pointer+m_dim);
   }
protected:
   Scalar* m_pointer;
   int m_dim;
}; // ----------------------------------------------------

template<typename Scalar, int cols>
Vector<Scalar,cols> operator+(const Vector<Scalar,cols>&, const RowVector<Scalar,cols>&);

template<typename Scalar, int cols>
Vector<Scalar,cols> operator+(const RowVector<Scalar,cols>&, const Vector<Scalar,cols>&);

template<typename Scalar, int cols>
Vector<Scalar,cols> operator+(const RowVector<Scalar,cols>&, const RowVector<Scalar,cols>&);



template<typename Scalar, int cols>
Vector<Scalar,cols> operator-(const Vector<Scalar,cols>&, const RowVector<Scalar,cols>&);

template<typename Scalar, int cols>
Vector<Scalar,cols> operator-(const RowVector<Scalar,cols>&, const Vector<Scalar,cols>&);

template<typename Scalar, int cols>
Vector<Scalar,cols> operator-(const RowVector<Scalar,cols>&, const RowVector<Scalar,cols>&);



template<typename Scalar, int cols>
Scalar operator*(const Vector<Scalar,cols>&, const RowVector<Scalar,cols>&);

template<typename Scalar, int cols>
Scalar operator*(const RowVector<Scalar,cols>&, const Vector<Scalar,cols>&);

template<typename Scalar, int cols>
Scalar operator*(const RowVector<Scalar,cols>&, const RowVector<Scalar,cols>&);



template<typename Scalar, int rows, int cols>
Vector<Scalar,rows> operator*(const Matrix<Scalar,rows,cols>&, const RowVector<Scalar,cols>&);

template<typename Scalar, int cols>
Vector<Scalar,cols> operator-(const RowVector<Scalar,cols>&);



template<typename Scalar, int cols>
Vector<Scalar,cols> operator*(const Scalar&, const RowVector<Scalar,cols>&);

template<typename Scalar, int cols>
Vector<Scalar,cols> operator*(const RowVector<Scalar,cols>&, const Scalar&);

template<typename Scalar, int cols>
Vector<Scalar,cols> operator/(const RowVector<Scalar,cols>&, const Scalar&);

template<typename Scalar, int cols>
std::ostream& operator << (std::ostream&, const RowVector<Scalar,cols>&);

#endif
